package net.bjzhanghao.osgi.example;

import javax.servlet.Servlet;

import org.eclipse.equinox.http.helper.BundleEntryHttpContext;
import org.eclipse.equinox.http.helper.ContextPathServletAdaptor;
import org.eclipse.equinox.jsp.jasper.JspServlet;
import org.osgi.framework.BundleActivator;
import org.osgi.framework.BundleContext;
import org.osgi.framework.ServiceReference;
import org.osgi.service.http.HttpContext;
import org.osgi.service.http.HttpService;
import org.osgi.util.tracker.ServiceTracker;

public class Activator implements BundleActivator {

	private ServiceTracker httpServiceTracker;

	String jspContext = "/jsps";
	String jspFolder = "/web_files";

	public void start(BundleContext context) throws Exception {
		httpServiceTracker = new HttpServiceTracker(context);
		httpServiceTracker.open();
	}

	public void stop(BundleContext context) throws Exception {
		httpServiceTracker.open();
	}

	private class HttpServiceTracker extends ServiceTracker {

		public HttpServiceTracker(BundleContext context) {
			super(context, HttpService.class.getName(), null);
		}

		public Object addingService(ServiceReference reference) {
			final HttpService httpService = (HttpService) context
					.getService(reference);
			try {
				HttpContext commonContext = new BundleEntryHttpContext(context
						.getBundle(), jspFolder);
				httpService.registerResources(jspContext, "/", commonContext);

				Servlet adaptedJspServlet = new ContextPathServletAdaptor(
						new JspServlet(context.getBundle(), jspFolder),
						jspContext);
				httpService.registerServlet(jspContext + "/*.jsp",
						adaptedJspServlet, null, commonContext);
			} catch (Exception e) {
				e.printStackTrace();
			}
			return httpService;
		}

		public void removedService(ServiceReference reference, Object service) {
			final HttpService httpService = (HttpService) service;
			httpService.unregister(jspContext);
			httpService.unregister(jspContext + "/*.jsp");
			super.removedService(reference, service);
		}
	}
}
